/**
 * @vitest-environment-options { "xmlPath": "./tests/resources/GP_Temperature_v3.2.0_DCC.xml" }
 */

import { select, toTextArr, toTextContentArr } from "../../util";
import { StatementMetaDataType, DCCDocument } from "../../../src";

const base = "/dcc:digitalCalibrationCertificate/dcc:administrativeData/dcc:statements";
const xpath = {
  statements: {
    statement1: {
      declaration: {
        content: `${base}/dcc:statement[1]/dcc:declaration/dcc:content`,
      },
      respAuthority: {
        name: {
          content: `${base}/dcc:statement[1]/dcc:respAuthority/dcc:name/dcc:content`,
        },
        email: `string(${base}/dcc:statement[1]/dcc:respAuthority/dcc:eMail)`,
        location: {
          city: `string(${base}/dcc:statement[1]/dcc:respAuthority/dcc:location/dcc:city)`,
          countryCode: `string(${base}/dcc:statement[1]/dcc:respAuthority/dcc:location/dcc:countryCode)`,
          postCode: `string(${base}/dcc:statement[1]/dcc:respAuthority/dcc:location/dcc:postCode)`,
        },
      },
      conformity: `string(${base}/dcc:statement[1]/dcc:conformity)`,
      data: {
        quantity: {
          measurementMetaData: {
            metaData1: {
              declaration: {
                content: `${base}/dcc:statement[1]/dcc:data/dcc:quantity/dcc:measurementMetaData/dcc:metaData[1]/dcc:declaration/dcc:content`,
              },
              validXMLList: `string(${base}/dcc:statement[1]/dcc:data/dcc:quantity/dcc:measurementMetaData/dcc:metaData[1]/dcc:validXMLList)`,
            },
          },
        },
      },
      refType: `string(${base}/dcc:statement[1]/@refType)`,
    },
    statement2: {
      declaration: {
        content: `${base}/dcc:statement[2]/dcc:declaration/dcc:content`,
      },
      date: `string(${base}/dcc:statement[2]/dcc:date)`,
      respAuthority: {
        name: {
          content: `${base}/dcc:statement[2]/dcc:respAuthority/dcc:name/dcc:content`,
        },
        email: `string(${base}/dcc:statement[2]/dcc:respAuthority/dcc:eMail)`,
        location: {
          city: `string(${base}/dcc:statement[2]/dcc:respAuthority/dcc:location/dcc:city)`,
          countryCode: `string(${base}/dcc:statement[2]/dcc:respAuthority/dcc:location/dcc:countryCode)`,
          postCode: `string(${base}/dcc:statement[2]/dcc:respAuthority/dcc:location/dcc:postCode)`,
        },
      },
      refType: `string(${base}/dcc:statement[2]/@refType)`,
    },
  },
};

describe("GP_Temperature_v3.2.0_DCC: StatementMetaDataType", () => {
  let dcc: DCCDocument, statement1, statement2: StatementMetaDataType, dom;

  beforeEach(() => {
    ({ dcc, dom } = recreateEnv());
    statement1 = dcc.digitalCalibrationCertificate.administrativeData.statements.statement[0];
    statement2 = dcc.digitalCalibrationCertificate.administrativeData.statements.statement[1];
  });

  test("should get correct statement 1 declaration content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.statements.statement1.declaration.content, dom);
    expect(toTextArr(statement1.declaration.content)).toEqual(toTextContentArr(expected));
  });

  /* responsible authority 1 */
  test("should get correct statement 1 responsible authority name content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.statements.statement1.respAuthority.name.content, dom);
    expect(toTextArr(statement1.respAuthority.name.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct statement 1 responsible authority email from XML", () => {
    expect(statement1.respAuthority.eMail._text).toBe(select(xpath.statements.statement1.respAuthority.email, dom));
  });

  test("should get correct statement 1 responsible authority location city from XML", () => {
    expect(statement1.respAuthority.location.city[0]._text).toBe(select(xpath.statements.statement1.respAuthority.location.city, dom));
  });
  test("should get correct statement 1 responsible authority location country code from XML", () => {
    expect(statement1.respAuthority.location.countryCode[0]._text).toBe(select(xpath.statements.statement1.respAuthority.location.countryCode, dom));
  });
  test("should get correct statement 1 responsible authority location post code from XML", () => {
    expect(statement1.respAuthority.location.postCode[0]._text).toBe(select(xpath.statements.statement1.respAuthority.location.postCode, dom));
  });

  test("should get correct statement 1 conformity from XML", () => {
    expect(statement1.conformity._text).toBe(select(xpath.statements.statement1.conformity, dom));
  });

  test("should get correct statement 1 metaData1 declaration content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.statements.statement1.data.quantity.measurementMetaData.metaData1.declaration.content, dom);
    expect(toTextArr(statement1.data.quantity[0].measurementMetaData.metaData[0].declaration.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct statement 1 metaData1 validXMLList from XML", () => {
    // get expected list from example xml
    expect(statement1.data.quantity[0].measurementMetaData.metaData[0].validXMLList._text).toBe(
      select(xpath.statements.statement1.data.quantity.measurementMetaData.metaData1.validXMLList, dom),
    );
  });

  test("should get correct statement 2 declaration content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.statements.statement2.declaration.content, dom);
    expect(toTextArr(statement2.declaration.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct statement 2 refType from XML", () => {
    expect(statement2.date._text).toBe(select(xpath.statements.statement2.date, dom));
  });

  /* responsible authority 2 */
  test("should get correct statement 2 responsible authority name content from XML", () => {
    // get expected list from example xml
    const expected = <Element[]>select(xpath.statements.statement2.respAuthority.name.content, dom);
    expect(toTextArr(statement2.respAuthority.name.content)).toEqual(toTextContentArr(expected));
  });

  test("should get correct statement 2 responsible authority email from XML", () => {
    expect(statement2.respAuthority.eMail._text).toBe(select(xpath.statements.statement2.respAuthority.email, dom));
  });

  test("should get correct statement 2 responsible authority location city from XML", () => {
    expect(statement2.respAuthority.location.city[0]._text).toBe(select(xpath.statements.statement2.respAuthority.location.city, dom));
  });
  test("should get correct statement 2 responsible authority location country code from XML", () => {
    expect(statement2.respAuthority.location.countryCode[0]._text).toBe(select(xpath.statements.statement2.respAuthority.location.countryCode, dom));
  });
  test("should get correct statement 2 responsible authority location post code from XML", () => {
    expect(statement2.respAuthority.location.postCode[0]._text).toBe(select(xpath.statements.statement2.respAuthority.location.postCode, dom));
  });

  test("should get correct statement 2 refType from XML", () => {
    expect(statement2._attr.refType).toBe(select(xpath.statements.statement2.refType, dom));
  });

  /* TODO: setters */
});
